# pylint: disable=missing-module-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-class-docstring
from enum import Enum


class LoggingHandler(Enum):

    """
    Standardized remote logging handlers for data engineering pipelines,
    designed for easy analysis and identification of remote logging
    requirements
    """
    
    NONE = "none"  # No remote handler
    LOCAL_STREAM = "local_stream"  # Local stream handler
    GCP_CLOUD_LOGGING = "gcp_cloud_logging"
    GCP_ERROR_REPORTING = "gcp_error_reporting"
    GCP_FIREBASE = "gcp_firebase"
    AWS_CLOUD_WATCH = "aws_cloud_watch"
    AZURE_MONITOR = "azure_monitor"
    AZURE_APPLICATION_INSIGHTS = "azure_application_insights"
    IBM_LOG_ANALYTICS = "ibm_log_analytics"
    ALIBABA_LOG_SERVICE = "alibaba_log_service"
    LOGGLY = "loggly"
    DATADOG = "datadog"
    NEW_RELIC = "new_relic"
    SENTRY = "sentry"
    SUMOLOGIC = "sumologic"
    # --- Other ---
    SYSLOG = "syslog" # For system logs
    CUSTOM = "custom" # For a user-defined remote handler
    OTHER = "other"

    def __str__(self):
        return self.value


class LogLevel(Enum):
    """
    Standardized notice levels for data engineering pipelines,
    designed for easy analysis and identification of manual 
    intervention needs.
    """
    ##############################
    # DEBUG (100 - 999)
    ##############################
    DEBUG = 100  # Detailed debug information (for development/troubleshooting)

    ##############################
    # INFO (1000 - 1999)
    ##############################
    INFO = 1000
    # Task Info (1110 - 1119)
    INFO_TASK_STARTED = 1110
    INFO_TASK_COMPLETE = 1111
    INFO_TASK_COMPLETE_WITH_NOTICES = 1112
    INFO_TASK_COMPLETE_WITH_WARNINGS = 1113
    INFO_TASK_PAUSED = 1114
    INFO_TASK_RESUMED = 1115
    INFO_TASK_CANCELLED = 1116
    INFO_TASK_STOPPED = 1117
    INFO_TASK_SKIPPED = 1118
    # Data Read Info (1120 - 1129)
    
    # INFO_READ_CLOUD_DB_COMPLETE = 1120
    # INFO_READ_LOCAL_DB_COMPLETE = 1121
    # INFO_READ_FILE_FROM_CLOUD_STORAGE_COMPLETE = 1122
    # INFO_READ_FILE_FROM_LOCAL_STORAGE_COMPLETE = 1123
    # INFO_READ_HTTP_GET_COMPLETE = 1124
    # INFO_READ_SECRET_FROM_SECRET_MANAGER_COMPLETE = 1125
    # # Data TRANSFORMATION Info (1130 - 1139)
    # INFO_FILTER_COMPLETE = 1130
    # INFO_TRANSFORM_COMPLETE = 1131

    # Subject Info (1210 - 1219)
    INFO_SUBJECT_STARTED = 1210

    # Subject Group Info (1220 - 1229)
    INFO_SUBJECT_GROUP_STARTED = 1220

    # Iteration Info (1310 - 1319)
    INFO_ITERATION_STARTED = 1310
    INFO_ITERATION_PAUSED = 1311
    INFO_ITERATION_RESUMED = 1312
    INFO_ITERATION_CANCELLED = 1313
    INFO_ITERATION_STOPPED = 1314
    INFO_ITERATION_SKIPPED = 1315
    # Pipeline Info (1410 - 1419)
    INFO_PIPELINE_STARTED = 1410
    INFO_PIPELINE_PAUSED = 1411
    INFO_PIPELINE_RESUMED = 1412
    INFO_PIPELINE_CANCELLED = 1413
    INFO_PIPELINE_STOPPED = 1414
    ##############################
    # READS (2000 - 2999)
    READ = 2000
    READ_COMPLETE = 2001
    READ_CLOUD_DB_COMPLETE = 2002
    READ_LOCAL_DB_COMPLETE = 2003
    READ_FILE_FROM_CLOUD_STORAGE_COMPLETE = 2004
    READ_FILE_FROM_LOCAL_STORAGE_COMPLETE = 2005
    READ_HTTP_GET_COMPLETE = 2006
    READ_SECRET_FROM_SECRET_MANAGER_COMPLETE = 2007

    ##############################
    # IN-MEMORY TASKS: TRANSOFRMATIONS/VALIDATIONS (3000 - 3999)
    IN_MEMORY_TASK = 3000
    # Validation (3010 - 3099)
    IN_MEMORY_VALIDATE_SCHEMA_COMPLETE = 3010
    IN_MEMORY_VALIDATE_COMPLETE = 3011
    # Sorting/Grouping/Joining (3100 - 3199)
    IN_MEMORY_SORT_COMPLETE = 3110
    IN_MEMORY_GROUP_BY_COMPLETE = 3111
    IN_MEMORY_JOIN_COMPLETE = 3112
    # Transformation (3200 - 3299)
    IN_MEMORY_TRANSFORM_COMPLETE = 3201
    IN_MEMORY_FILTER_COMPLETE = 3202
    IN_MEMORY_AGGREGATE_COMPLETE = 3203
    IN_MEMORY_NORMALIZE_COMPLETE = 3204
    IN_MEMORY_DENORMALIZE_COMPLETE = 3205
    IN_MEMORY_DEDUPLICATE_COMPLETE = 3206
    IN_MEMORY_ENRICH_COMPLETE = 3207
    IN_MEMORY_DECRYPT_COMPLETE = 3208
    IN_MEMORY_ENCRYPT_COMPLETE = 3209
    IN_MEMORY_MASK_COMPLETE = 3210
    IN_MEMORY_FILL_MISSING_VALUES_COMPLETE = 3211
    IN_MEMORY_REPLACE_VALUES_COMPLETE = 3212

    ##############################
    # ACTIONS (4100 - 4999)
    ##############################
    ACTION = 4000 # General action, no immediate action required
    # General Action Completions (4100 - 4199)
    ACTION_CWUD_COMPLETE = 4100 # General action, no immediate action required
    ACTION_CREATE_COMPLETE = 4101
    ACTION_WRITE_COMPLETE = 4102
    ACTION_UPDATE_COMPLETE = 4103
    ACTION_DELETE_COMPLETE = 4104
    # Bulk Action Completions (4200 - 4299)
    ACTION_CWUD_BULK_COMPLETE = 4200
    ACTION_CREATE_BULK_COMPLETE = 4201
    ACTION_WRITE_BULK_COMPLETE = 4202
    ACTION_UPDATE_BULK_COMPLETE = 4203
    ACTION_DELETE_BULK_COMPLETE = 4204
    # Cloud Storage Actions (4300 - 4399)
    ACTION_WRITE_IN_CLOUD_STORAGE_COMPLETE = 4300
    ACTION_UPDATE_IN_CLOUD_STORAGE_COMPLETE = 4301
    ACTION_DELETE_IN_CLOUD_STORAGE_COMPLETE = 4302
    ACTION_WRITE_BULK_IN_CLOUD_STORAGE_COMPLETE = 4310
    ACTION_UPDATE_BULK_IN_CLOUD_STORAGE_COMPLETE = 4311
    ACTION_DELETE_BULK_IN_CLOUD_STORAGE_COMPLETE = 4312
    # Local Storage Actions (4400 - 4499)
    ACTION_WRITE_IN_LOCAL_STORAGE_COMPLETE = 4400
    ACTION_UPDATE_IN_LOCAL_STORAGE_COMPLETE = 4401
    ACTION_DELETE_IN_LOCAL_STORAGE_COMPLETE = 4402
    # Local DB Actions (4500 - 4599)
    ACTION_WRITE_IN_LOCAL_DB_COMPLETE = 4500
    ACTION_UPDATE_IN_LOCAL_DB_COMPLETE = 4501
    ACTION_DELETE_IN_LOCAL_DB_COMPLETE = 4502
    # Cloud DB Actions (4600 - 4699)
    ACTION_WRITE_IN_CLOUD_DB_COMPLETE = 4600
    ACTION_UPDATE_IN_CLOUD_DB_COMPLETE = 4601
    ACTION_DELETE_IN_CLOUD_DB_COMPLETE = 4602
    ACTION_MERGE_IN_CLOUD_DB_COMPLETE = 4603
    # Bulk Cloud DB Actions (4610 - 4699)
    ACTION_WRITE_BULK_IN_CLOUD_DB_COMPLETE = 4610
    ACTION_UPDATE_BULK_IN_CLOUD_DB_COMPLETE = 4611
    ACTION_DELETE_BULK_IN_CLOUD_DB_COMPLETE = 4612
    ACTION_MERGE_BULK_IN_CLOUD_DB_COMPLETE = 4613
    # Cloud  Create Actions (4620 - 4699)
    ACTION_CREATE_CLOUD_DB_TABLE_COMPLETE = 4620
    ACTION_CREATE_CLOUD_DB_COLLECTION_COMPLETE = 4621
    ACTION_CREATE_CLOUD_DB_DOCUMENT_COMPLETE = 4622
    ACTION_CREATE_AND_LOAD_CLOUD_DB_TABLE_COMPLETE = 4623

    # Cloud DB Delete Actions (4630 - 4699)
    ACTION_DELETE_CLOUD_DB_TABLE_COMPLETE = 4630
    ACTION_DELETE_CLOUD_DB_COLLECTION_COMPLETE = 4631
    ACTION_DELETE_CLOUD_DB_DOCUMENT_COMPLETE = 4632
    # HTTP Actions (4700 - 4799)
    ACTION_WRITE_HTTP_POST_COMPLETE = 4700
    ACTION_WRITE_HTTP_PUT_COMPLETE = 4701
    ACTION_WRITE_HTTP_PATCH_COMPLETE = 4702
    ACTION_WRITE_HTTP_DELETE_COMPLETE = 4703


    ##############################
    # SUCCESSES (5000 - 5499)
    ##############################

    SUCCESS = 5000
    # Task Successes (5100 - 5199)
    SUCCESS_TASK_COMPLETE = 5100
    SUCCESS_TASK_COMPLETE_WITH_NOTICES = 5101
    SUCCESS_TASK_COMPLETE_WITH_WARNINGS = 5102

    # Subject Successes (5200 - 5299)
    SUCCESS_SUBJECT_COMPLETE = 5200
    SUCCESS_SUBJECT_COMPLETE_WITH_NOTICES = 5201
    SUCCESS_SUBJECT_COMPLETE_WITH_WARNINGS = 5202

    SUCCESS_SUBJECT_GROUP_COMPLETE = 5203
    SUCCESS_SUBJECT_GROUP_COMPLETE_WITH_NOTICES = 5204
    SUCCESS_SUBJECT_GROUP_COMPLETE_WITH_WARNINGS = 5205


    # Iteration Successes (5300 - 5399)
    SUCCESS_ITERATION_COMPLETE = 5300
    SUCCESS_ITERATION_COMPLETE_WITH_NOTICES = 5301
    SUCCESS_ITERATION_COMPLETE_WITH_WARNINGS = 5302

    # Pipeline Successes (5400 - 5499)
    SUCCESS_PIPELINE_COMPLETE = 5400
    SUCCESS_PIPELINE_COMPLETE_WITH_NOTICES = 5401
    SUCCESS_PIPELINE_COMPLETE_WITH_WARNINGS = 5402

    ##############################
    # NOTICES (6000 - 6199)
    ##############################
    NOTICE = 6000
    NOTICE_ALREADY_EXISTS = 6100
    NOTICE_PARTIAL_EXISTS = 6101

    NOTICE_FILE_IN_CLOUD_STORAGE_ALREADY_EXISTS = 6110
    NOTICE_DATA_IN_CLOUD_DB_ALREADY_EXISTS = 6111
    NOTICE_DATA_IN_CLOUD_DB_PARTIALLY_EXISTS = 6112
    NOTICE_TABLE_DOESNT_EXIST = 6113
    NOTICE_COLLECTION_DOESNT_EXIST = 6114
    NOTICE_DOCUMENT_DOESNT_EXIST = 6115

    ##############################
    # WARNINGS (7000 - 7299)
    ##############################
    WARNING = 7000
    WARNING_REVIEW_RECOMMENDED = 7100
    WARNING_DATA_SCHEMA_ISSUE = 7200
    WARNING_METADATA_SCHEMA_ISSUE = 7201

    ##############################
    # ERRORS (8000 - 8999)
    ##############################
    ERROR = 8000
    ERROR_EXCEPTION = 8001
    ERROR_CUSTOM = 8002

    ERROR_DATA_QUALITY_ISSUE = 8003
    ERROR_VALIDATION_ISSUE = 8004
    ERROR_SCHEMA_ISSUE = 8005
    ERROR_SCHEMA_VALIDATION_ISSUE = 8006

    # Read Errors (8200 - 8299)
    ERROR_READ_FAILED = 8201
    ERROR_READ_PARTIALLY_FAILED = 8202
    ERROR_READ_DB_FAILED = 8210
    ERROR_READ_DB_WITH_ERRORS = 8211
    ERROR_READ_FILE_FROM_CLOUD_STORAGE_FAILED = 8220
    ERROR_READ_FILE_FROM_CLOUD_STORAGE_PARTIALLY_FAILED = 8221
    ERROR_READ_FILE_FROM_LOCAL_STORAGE_FAILED = 8230
    ERROR_READ_FILE_FROM_LOCAL_STORAGE_PARTIALLY_FAILED = 8231
    # HTTP Read Errors (8240 - 8299)
    ERROR_READ_HTTP_GET_FAILED = 8250
    ERROR_READ_HTTP_GET_PARTIALLY_FAILED = 8251
    ERROR_READ_HTTP_GET_WITH_ERRORS = 8252
    ERROR_READ_HTTP_GET_WITH_WARNINGS_OR_ERRORS = 8253
    # Secret Manager Read Errors (8260 - 8299)
    ERROR_READ_SECRET_FROM_SECRET_MANAGER_FAILED = 8260

    # In-Memory Task Errors (8300 - 8399)
    ERROR_IN_MEMORY_TASK_FAILED = 8310
    ERROR_IN_MEMORY_TASK_COMPLETE_WITH_ERRORS = 8311
    ERROR_IN_MEMORY_TRANSFORMATION_FAILED= 8312
    ERROR_IN_MEMORY_TRANSFORMATION_COMPLETE_WITH_ERRORS = 8313

    ERROR_IN_MEMORY_VALIDATE_FAILED = 8351
    ERROR_IN_MEMORY_VALIDATE_SCHEMA_FAILED = 8352
    

    # Action Errors (8400 - 8499)
    ERROR_ACTION_PARTIALLY_FAILED = 8400
    ERROR_ACTION_FAILED = 8401
    ERROR_ACTION_WITH_ERRORS = 8402
    ERROR_ACTION_WITH_WARNINGS_OR_ERRORS = 8403
    # General Data Persistence Errors (8410 - 8429)
    ERROR_ACTION_CWUD_FAILED = 8420
    ERROR_ACTION_CREATE_FAILED = 8421
    ERROR_ACTION_WRITE_FAILED = 8422
    ERROR_ACTION_UPDATE_FAILED = 8423
    ERROR_ACTION_DELETE_FAILED = 8424
    ERROR_ACTION_MERGE_FAILED = 8425
    # Data Persistence Errors with Warnings (8430 - 8449)
    ERROR_ACTION_CWUD_WITH_ERRORS = 8440
    ERROR_ACTION_CREATE_WITH_ERRORS = 8441
    ERROR_ACTION_WRITE_WITH_ERRORS = 8442
    ERROR_ACTION_UPDATE_WITH_ERRORS = 8443
    ERROR_ACTION_DELETE_WITH_ERRORS = 8444
    ERROR_ACTION_MERGE_WITH_ERRORS = 8445
    # Cloud Storage Errors (8450 - 8459)
    ERROR_ACTION_WRITE_IN_CLOUD_STORAGE_FAILED = 8450
    ERROR_ACTION_UPDATE_IN_CLOUD_STORAGE_FAILED = 8451
    ERROR_ACTION_DELETE_IN_CLOUD_STORAGE_FAILED = 8452
    # Local Storage Errors (8460 - 8469)
    ERROR_ACTION_WRITE_IN_LOCAL_STORAGE_FAILED = 8460
    ERROR_ACTION_UPDATE_IN_LOCAL_STORAGE_FAILED = 8461
    ERROR_ACTION_DELETE_IN_LOCAL_STORAGE_FAILED = 8462
    # Local Database Errors (8470 - 8474)
    ERROR_ACTION_WRITE_IN_LOCAL_DB_FAILED = 8470
    ERROR_ACTION_UPDATE_IN_LOCAL_DB_FAILED = 8471
    ERROR_ACTION_DELETE_IN_LOCAL_DB_FAILED = 8472
    # Local Database Errors with Warnings (8475 - 8479)
    ERROR_ACTION_WRITE_IN_LOCAL_DB_WITH_ERRORS = 8475
    ERROR_ACTION_UPDATE_IN_LOCAL_DB_WITH_ERRORS = 8476
    ERROR_ACTION_DELETE_IN_LOCAL_DB_WITH_ERRORS = 8477
    # Cloud Database Errors (8480 - 8484)
    ERROR_ACTION_WRITE_IN_CLOUD_DB_FAILED = 8480
    ERROR_ACTION_UPDATE_IN_CLOUD_DB_FAILED = 8481
    ERROR_ACTION_DELETE_IN_CLOUD_DB_FAILED = 8482
    ERROR_ACTION_MERGE_IN_CLOUD_DB_FAILED = 8483
    # Cloud Database Errors with Warnings (8485 - 8489)
    ERROR_ACTION_WRITE_IN_CLOUD_DB_WITH_ERRORS = 8485
    ERROR_ACTION_UPDATE_IN_CLOUD_DB_WITH_ERRORS = 8486
    ERROR_ACTION_DELETE_IN_CLOUD_DB_WITH_ERRORS = 8487
    ERROR_ACTION_MERGE_IN_CLOUD_DB_WITH_ERRORS = 8488
    # Cloud Database Errors (8490 - 8494)
    ERROR_ACTION_CREATE_CLOUD_DB_TABLE_FAILED = 8490
    ERROR_ACTION_CREATE_CLOUD_DB_COLLECTION_FAILED = 8491
    ERROR_ACTION_CREATE_CLOUD_DB_DOCUMENT_FAILED = 8492
    ERROR_ACTION_DELETE_CLOUD_DB_TABLE_FAILED = 8493
    ERROR_ACTION_DELETE_CLOUD_DB_COLLECTION_FAILED = 8494
    ERROR_ACTION_DELETE_CLOUD_DB_DOCUMENT_FAILED = 8495
    ERROR_ACTION_CREATE_AND_LOAD_CLOUD_DB_TABLE_FAILED = 8496

    # Threshold Errors (8800 - 8899)
    ERROR_TASK_THRESHOLD_REACHED = 8800
    ERROR_DATA_QUALITY_THRESHOLD_REACHED = 8801
    ERROR_METADATA_QUALITY_THRESHOLD_REACHED = 8802
    
    ERROR_SUBJECT_THRESHOLD_REACHED = 8811
    ERROR_ITERATION_THRESHOLD_REACHED = 8812
    ERROR_PIPELINE_THRESHOLD_REACHED = 8813

    ##############################
    # FAILURES (9000 - 9899)
    ##############################
    FAILED = 9000

    # Task Failures (9100 - 9199)
    FAILED_TASK = 9100
    FAILED_TASK_COMPLETE_WITH_ERRORS = 9101

    # Subject Failures (9200 - 9299)
    FAILED_SUBJECT = 9200
    FAILED_SUBJECT_COMPLETE_WITH_ERRORS = 9201
    FAILED_SUBJECT_GROUP = 9202
    FAILED_SUBJECT_GROUP_COMPLETE_WITH_ERRORS = 9203

    # Iteration Failures (9300 - 9349)
    FAILED_ITERATION = 9300
    FAILED_ITERATION_COMPLETE_WITH_ERRORS = 9301

    # Pipeline Failures (9400 - 9499)
    FAILED_PIPELINE_COMPLETE_WITH_ERRORS = 9400
    FAILED_PIPELINE_EARLY_EXITED = 9401

    # Critical System Failures (9800 - 9899)
    FAILED_CRITICAL_SYSTEM_FAILURE = 9800

    def __str__(self):
        return self.value